local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Group = import('/lua/maui/group.lua').Group
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local Button = import('/lua/maui/button.lua').Button
local borders = import('/lua/ui/game/borders.lua').GetMapGroup()
local ToolTip = import('/lua/ui/game/tooltip.lua')
local EffectHelpers = import('/lua/maui/effecthelpers.lua')

--should squads be automatically selected?
AutoSelection = true
local panel = false
local mouseover = false

function CreatePanel(parent)
	local panel = Bitmap(parent)
	panel:SetTexture('/mods/advancedgrouping/textures/panel.dds')
	panel.Right:Set(function() return parent.Right() - 270 end)
	panel.Top:Set(function() return parent.Top() + 5 end)
	panel.Depth:Set(100)
	panel.HandleEvent = function(self, event)
		if event.Type == 'MouseExit' then
			CloseToolTip()
		end
	end
	return panel
end

function CreateSquadButton(parent)
	local button = Button(parent,
		'/mods/advancedgrouping/textures/splitbtn_up.dds',
		'/mods/advancedgrouping/textures/splitbtn_down.dds',
		'/mods/advancedgrouping/textures/splitbtn_over.dds',
		'/mods/advancedgrouping/textures/splitbtn_up.dds')
	button.title = UIUtil.CreateText(button, "C", 10)
	button.title:SetColor('338833')
	LayoutHelpers.AtCenterIn(button.title, button, 2, 1)
	button.OnClick = function(self)
		import("/mods/advancedgrouping/selectionsplit.lua").SquadCreator()
	end
	button.oldHandleEvent = button.HandleEvent
	button.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Create Squad", Body = "Create a squad from the current selection."})
		end
		button.oldHandleEvent(self, event)
	end
	return button
end

function DisbandSquadButton(parent)
	local button = Button(parent,
		'/mods/advancedgrouping/textures/splitbtn_up.dds',
		'/mods/advancedgrouping/textures/splitbtn_down.dds',
		'/mods/advancedgrouping/textures/splitbtn_over.dds',
		'/mods/advancedgrouping/textures/splitbtn_up.dds')
	button.title = UIUtil.CreateText(button, "X", 10)
	button.title:SetColor('883333')
	LayoutHelpers.AtCenterIn(button.title, button, 2, 1)
	button.OnClick = function(self)
		import("/mods/advancedgrouping/selectionsplit.lua").Disband()
	end
	button.oldHandleEvent = button.HandleEvent
	button.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Disband Squads", Body = "Disband any currently selected squads."})
		end
		button.oldHandleEvent(self, event)
	end
	return button
end


function AutoSelectButton(parent)
	local button = Checkbox(parent,
		'/mods/advancedgrouping/textures/autoselectbtn_up.dds',
		'/mods/advancedgrouping/textures/autoselectbtn_up.dds',
		'/mods/advancedgrouping/textures/autoselectbtn_over.dds',
		'/mods/advancedgrouping/textures/autoselectbtn_over.dds',
		'/mods/advancedgrouping/textures/autoselectbtn_up.dds',
		'/mods/advancedgrouping/textures/autoselectbtn_up.dds')
	button.title = UIUtil.CreateText(button, "Squads", 9)
	LayoutHelpers.AtCenterIn(button.title, button, 1, 0)
	button.OnCheck = function(self, checked)
		if checked then
			AutoSelection = true
			button.title:SetColor('2ec74d')
		else
			AutoSelection = false
			button.title:SetColor('e61b1b')
		end
	end
	button.oldHandleEvent = button.HandleEvent
	button.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Auto Squad Selection", Body = "Toggle automatic selection of squads when any part of a squad is selected."})

		end
		button.oldHandleEvent(self, event)
	end
	button:SetCheck(true)
	return button
end

function CreateSquadControls(parent)
	local squadgroup = Group(parent)
	squadgroup.Left:Set(parent.Left)
	squadgroup.Top:Set(parent.Top)
	squadgroup.Right:Set(parent.Right)
	squadgroup.Height:Set(20)
	local squadbtn = CreateSquadButton(squadgroup)
	local disbandbtn = DisbandSquadButton(squadgroup)
	local autoselectbtn = AutoSelectButton(squadgroup)
	LayoutHelpers.AtLeftTopIn(squadbtn, squadgroup, 43, 2)
	LayoutHelpers.AtLeftTopIn(disbandbtn, squadgroup, 60, 2)
	LayoutHelpers.AtLeftTopIn(autoselectbtn, squadgroup, 3, 2)
	return squadgroup
end

function CreateSplitButtons(parent)
	local btngroup = Group(parent)
	btngroup.Height:Set(20)
	btngroup.Width:Set(80)
	local btn = {}
	for i = 2, 5 do
		local index = i
		btn[index] = Button(btngroup,
			'/mods/advancedgrouping/textures/splitbtn_up.dds',
			'/mods/advancedgrouping/textures/splitbtn_down.dds',
			'/mods/advancedgrouping/textures/splitbtn_over.dds',
			'/mods/advancedgrouping/textures/splitbtn_up.dds')
		if index == 2 then
			LayoutHelpers.AtLeftTopIn(btn[index], btngroup)
		else
			LayoutHelpers.RightOf(btn[index], btn[index-1], 1)
		end
		btn[index].title = UIUtil.CreateText(btn[index], i, 10)
		LayoutHelpers.AtCenterIn(btn[index].title, btn[index], 2, 1)
		btn[index].OnClick = function(self)
			import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(index)
		end
		btn[index].oldHandleEvent = btn[index].HandleEvent
		btn[index].HandleEvent = function(self, event)
			if event.Type == 'MouseEnter' then
				CreateToolTip(self, {Title = "Split Selection", Body = "Split current selection into "..index.." subgroups."})
			end
			btn[index].oldHandleEvent(self, event)
		end
	end
	return btngroup
end

function CreateSplitControls(parent)
	local splitgroup = Group(parent)
	splitgroup.Left:Set(parent.Left)
	splitgroup.Top:Set(function() return parent.Top() + 20 end)
	splitgroup.Right:Set(parent.Right)
	splitgroup.Height:Set(30)
	local title = UIUtil.CreateText(splitgroup, "Selection split:", 9)
	title.HandleEvent = function(self, event)
		if event.Type == 'MouseEnter' then
			CreateToolTip(self, {Title = "Split Selection", Body = "Click a numbered button to split the current selection into X subgroups."})
		end
	end
	local buttons = CreateSplitButtons(splitgroup)
	LayoutHelpers.AtLeftTopIn(title, splitgroup, 5, 1)
	LayoutHelpers.AtLeftTopIn(buttons, splitgroup, 3, 11)
	return splitgroup
end

function CreateToolTip(control, data, togglebutton)
	if mouseover then mouseover:Destroy() mouseover = false end
	local text = data.Title
	local body = data.Body
	mouseover = ToolTip.CreateExtendedToolTip(control, text, body)
	mouseover.Top:Set(function() return panel.Top() + 2 end)
	mouseover.Right:Set(function() return panel.Left() - 1 end)
	mouseover:SetAlpha(0, true)
	mouseover:SetNeedsFrameUpdate(true)
	
	local delayTime = 0.5
	local totaltime = 0
	mouseover.OnFrame = function(self, deltaTime)
		if totaltime > delayTime then
			local alpha = self:GetAlpha() + (deltaTime * 4)
			if alpha < 1 then
				self:SetAlpha(alpha, true)
			else
				self:SetAlpha(1, true)
				self:SetNeedsFrameUpdate(false)
			end
		end
		totaltime = totaltime + deltaTime
	end
end
		
function CloseToolTip()
	if mouseover then mouseover:Destroy() mouseover = false end
end

function hideEffect(control, subcontrol1, subcontrol2)
	control:DisableHitTest(true)
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.1
	local time2 = 0.2
	local minHeight = 16
    local timeAccum = 0

	EffectHelpers.FadeOut(subcontrol1, timefade)
	EffectHelpers.FadeOut(subcontrol2, timefade)
	
    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
            self:SetNeedsFrameUpdate(false)
            self:Hide()
			self.Left:Set(function() return self.Right() - 3 end)
		elseif timeAccum >= time1 then
			self.Left:Set(function() return self.Right() - self.BitmapWidth()*((time2+time1 - timeAccum)/(time2)) end)
			self.Bottom:Set(function() return self.Top() + minHeight end)
		else
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((time1 - timeAccum)/(time1)) end )
		end
    end
end

function showEffect(control, subcontrol1, subcontrol2)
	control:Show()
    control:SetNeedsFrameUpdate(true)

	local timefade = 0.1
	local time1 = 0.2
	local time2 = 0.1
	local minHeight = 16
    local timeAccum = 0

    control.OnFrame = function(self, elapsedTime)
        timeAccum = timeAccum + elapsedTime
		if timeAccum >= (time1 + time2) then
			self:EnableHitTest(true)
            self:SetNeedsFrameUpdate(false)
			LayoutHelpers.ResetLeft(self)
			LayoutHelpers.ResetBottom(self)
			EffectHelpers.FadeIn(subcontrol1, timefade)
			EffectHelpers.FadeIn(subcontrol2, timefade)
		elseif timeAccum >= time1 then
			self.Bottom:Set(function() return self.Top() + minHeight + (self.BitmapHeight() - minHeight)*((timeAccum - time1)/(time2)) end )
			self.Left:Set(function() return self.Right() - self.BitmapWidth() end)
        else
			self.Left:Set(function() return self.Right() - self.BitmapWidth()*((timeAccum)/(time1)) end)
        end
    end
end

function Init()
	panel = CreatePanel(borders)
	squadPanel = CreateSquadControls(panel)
	splitPanel = CreateSplitControls(panel)
	panelToggle = UIUtil.CreateCheckbox(borders, 
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_up.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_over.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-close_btn_dis.dds'),
        UIUtil.UIFile('/game/avatar-tab-btn/tab-open_btn_dis.dds'))
    LayoutHelpers.AtRightTopIn(panelToggle, borders, 3, 5)
	panelToggle.Height:Set(16)
	panelToggle.Depth:Set(100)
    panelToggle.OnCheck = function(self, checked)
        if checked then
			hideEffect(panel, squadPanel, splitPanel)
		else
			showEffect(panel, squadPanel, splitPanel)
		end
    end
	panelToggle.oldHandleEvent = panelToggle.HandleEvent
	panelToggle.HandleEvent = function(self, event)
		if event.Type == 'ButtonPress' then
			CloseToolTip()
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the advanced selection panel."}, true)
			else
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the advanced selection panel."}, true)
			end
		elseif event.Type == 'MouseEnter' then
			if self:IsChecked() then
				CreateToolTip(self, {Title = "Maximize", Body = "Maximize the advanced selection panel."}, true)
			else
				CreateToolTip(self, {Title = "Minimize", Body = "Minimize the advanced selection panel."}, true)
			end
		elseif event.Type == 'MouseExit' then
			CloseToolTip()
		end
		panelToggle.oldHandleEvent(self, event)
	end
	
	
	local newSelectionsMap = {
    ['Alt-1']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt1")'},
    ['Alt-2']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt2")'},
    ['Alt-3']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt3")'},
    ['Alt-4']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt4")'},
    ['Alt-5']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt5")'},
    ['Alt-6']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt6")'},
    ['Alt-7']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt7")'},
    ['Alt-8']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt8")'},
    ['Alt-9']              = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SplitSelector("alt9")'},
	['G']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").GroupChecker()'},
	['Ctrl-G']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SquadCreator()'},
	['Ctrl-D']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").Disband()'},
#	['Alt-X']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SquadCheck()'},
	['Ctrl-Alt-1']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(1)'},
	['Ctrl-Alt-2']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(2)'},
	['Ctrl-Alt-3']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(3)'},
	['Ctrl-Alt-4']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(4)'},
	['Ctrl-Alt-5']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(5)'},
	['Ctrl-Alt-6']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(6)'},
	['Ctrl-Alt-7']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(7)'},
	['Ctrl-Alt-8']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(8)'},
	['Ctrl-Alt-9']         = {action =  'UI_Lua import("/mods/advancedgrouping/selectionsplit.lua").SelectionSplitter(9)'},
	}
	IN_AddKeyMapTable(newSelectionsMap)
end
